@@ -15,6 +15,24 @@
  *
  * You should have received a copy of the GNU General Public License along with
  * OpenPACE.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Additional permission under GNU GPL version 3 section 7
+ *
+ * If you modify this Program, or any covered work, by linking or combining it
+ * with OpenSSL (or a modified version of that library), containing
+ * parts covered by the terms of OpenSSL's license, the licensors of
+ * this Program grant you additional permission to convey the resulting work.
+ * Corresponding Source for a non-source form of such a combination shall include
+ * the source code for the parts of OpenSSL used as well as that of the
+ * covered work.
+ *
+ * If you modify this Program, or any covered work, by linking or combining it
+ * with OpenSC (or a modified version of that library), containing
+ * parts covered by the terms of OpenSC's license, the licensors of
+ * this Program grant you additional permission to convey the resulting work. 
+ * Corresponding Source for a non-source form of such a combination shall include
+ * the source code for the parts of OpenSC used as well as that of the
+ * covered work.
  */
 
 /**
@@ -224,18 +242,6 @@ ASN1_ITEM_TEMPLATE(CVC_CERT) =
 ASN1_ITEM_TEMPLATE_END(CVC_CERT)
 IMPLEMENT_ASN1_FUNCTIONS(CVC_CERT)
 
-ASN1_ADB_TEMPLATE(cert_def) = ASN1_SIMPLE(CVC_CERTIFICATE_DESCRIPTION, termsOfUsage.other, ASN1_ANY);
-
-ASN1_ADB(CVC_CERTIFICATE_DESCRIPTION) = {
-#ifdef HAVE_PATCHED_OPENSSL
-        ADB_ENTRY(NID_id_plainFormat, ASN1_IMP(CVC_CERTIFICATE_DESCRIPTION, termsOfUsage.plainTerms, ASN1_UTF8STRING, 0x05)),
-        ADB_ENTRY(NID_id_htmlFormat, ASN1_IMP(CVC_CERTIFICATE_DESCRIPTION, termsOfUsage.htmlTerms, ASN1_IA5STRING, 0x05)),
-        ADB_ENTRY(NID_id_pdfFormat, ASN1_IMP(CVC_CERTIFICATE_DESCRIPTION, termsOfUsage.pdfTerms, ASN1_OCTET_STRING, 0x05))
-#else
-        ADB_ENTRY(NID_undef, ASN1_IMP(CVC_CERTIFICATE_DESCRIPTION, termsOfUsage.unused, ASN1_OCTET_STRING, 0x05))
-#endif
-} ASN1_ADB_END(CVC_CERTIFICATE_DESCRIPTION, 0, descriptionType, 0, &cert_def_tt, NULL);
-
 ASN1_SEQUENCE(CVC_COMMCERT_SEQ) = {
         ASN1_SET_OF(CVC_COMMCERT_SEQ, values, ASN1_OCTET_STRING),
 } ASN1_SEQUENCE_END(CVC_COMMCERT_SEQ)
@@ -245,7 +251,7 @@ ASN1_SEQUENCE(CVC_CERTIFICATE_DESCRIPTION) = {
         ASN1_IMP_OPT(CVC_CERTIFICATE_DESCRIPTION, issuerURL, ASN1_PRINTABLESTRING, 0x02),
         ASN1_IMP(CVC_CERTIFICATE_DESCRIPTION, subjectName, ASN1_UTF8STRING, 0x03),
         ASN1_IMP_OPT(CVC_CERTIFICATE_DESCRIPTION, subjectURL, ASN1_PRINTABLESTRING, 0x04),
-        ASN1_ADB_OBJECT(CVC_CERTIFICATE_DESCRIPTION),
+        ASN1_IMP_OPT(CVC_CERTIFICATE_DESCRIPTION, termsOfUsage, ASN1_OCTET_STRING, 0x05),
         ASN1_IMP_OPT(CVC_CERTIFICATE_DESCRIPTION, redirectURL, ASN1_PRINTABLESTRING, 0x06),
         ASN1_IMP_OPT(CVC_CERTIFICATE_DESCRIPTION, commCertificates, CVC_COMMCERT_SEQ, 0x07),
 } ASN1_SEQUENCE_END(CVC_CERTIFICATE_DESCRIPTION)
@@ -294,7 +300,7 @@ ASN1_SEQUENCE(CVC_CERT_AUTHENTICATION_REQUEST_SEQ) = {
 
 /* Change the tag of the Authentication Request to 0x67 */
 ASN1_ITEM_TEMPLATE(CVC_CERT_AUTHENTICATION_REQUEST) =
-    ASN1_EX_TEMPLATE_TYPE(ASN1_TFLG_SEQUENCE_OF|ASN1_TFLG_IMPTAG|ASN1_TFLG_APPLICATION, 7, CVC_CERT_AUTHENTICATION_REQUEST, CVC_CERT_AUTHENTICATION_REQUEST_SEQ)
+    ASN1_EX_TEMPLATE_TYPE(ASN1_TFLG_IMPTAG|ASN1_TFLG_APPLICATION, 7, CVC_CERT_AUTHENTICATION_REQUEST, CVC_CERT_AUTHENTICATION_REQUEST_SEQ)
 ASN1_ITEM_TEMPLATE_END(CVC_CERT_AUTHENTICATION_REQUEST)
 IMPLEMENT_ASN1_FUNCTIONS(CVC_CERT_AUTHENTICATION_REQUEST)
 
@@ -376,8 +382,7 @@ CVC_d2i_CVC_CERT(CVC_CERT **cert, const unsigned char **in, long len)
         check(ret->body->public_key->cont1 && ret->body->public_key->cont2,
                 "public key missing");
     } else {
-        log_err("unknown credentials in certificate");
-        goto err;
+        debug("unknown credentials in certificate");
     }
 
     return ret;
@@ -584,13 +589,11 @@ CVC_pubkey2eckey(int all_parameters, const CVC_PUBKEY *public_key,
                     public_key->cont6,
                     public_key->cont7,
                     bn_ctx)) {
-            EC_KEY_free(ec);
             log_err("Internal error");
             goto err;
         }
 
         ok = EVP_PKEY_set1_EC_KEY(key, ec);
-        EC_KEY_free(ec);
     } else {
         /* If cert is not a CVCA certificate it MUST NOT contain any domain
          * parameters. We take the domain parameters from the domainParameters
@@ -607,7 +610,7 @@ CVC_pubkey2eckey(int all_parameters, const CVC_PUBKEY *public_key,
         check(EVP_PKEY_base_id(key) == EVP_PKEY_EC,
                "Incorrect domain parameters");
 
-        ec = (EC_KEY *) EVP_PKEY_get0(key);
+        ec = EVP_PKEY_get1_EC_KEY(key);
         check(ec, "Failed to extract domain parameters");
 
         group = EC_KEY_get0_group(ec);
@@ -627,6 +630,8 @@ CVC_pubkey2eckey(int all_parameters, const CVC_PUBKEY *public_key,
 err:
     if (point)
         EC_POINT_free(point);
+    if (ec)
+        EC_KEY_free(ec);
 
     return ok;
 }
@@ -681,14 +686,10 @@ CVC_print(BIO *bio, const CVC_CERT *cv, int indent)
     r = 1;
 
 err:
-    if (effective_date)
-        OPENSSL_free(effective_date);
-    if (expiration_date)
-        OPENSSL_free(expiration_date);
-    if (car)
-        OPENSSL_free(car);
-    if (chr)
-        OPENSSL_free(chr);
+    free(effective_date);
+    free(expiration_date);
+    free(car);
+    free(chr);
 
     return r;
 }
@@ -861,10 +862,8 @@ int certificate_request_print(BIO *bio,
     r = 1;
 
 err:
-    if (car)
-        OPENSSL_free(car);
-    if (chr)
-        OPENSSL_free(chr);
+    free(car);
+    free(chr);
 
     return r;
 }
@@ -893,8 +892,7 @@ int certificate_authentication_request_print(BIO *bio,
     r = 1;
 
 err:
-    if (car)
-        OPENSSL_free(car);
+    free(car);
 
     return r;
 }
@@ -1031,7 +1029,7 @@ cvc_get_reference_string(ASN1_OCTET_STRING *ref)
     check(ref, "Invalid input");
     check(is_chr(ref->data, ref->length), "Invalid certificate reference");
 
-    ret = OPENSSL_malloc(ref->length + 1);
+    ret = malloc(ref->length + 1);
     check(ret, "Not enough memory");
 
     memcpy(ret, ref->data, ref->length);
@@ -1051,7 +1049,7 @@ cvc_get_date_string(ASN1_OCTET_STRING *date)
             || !is_bcd(date->data, date->length))
         return NULL;
 
-    ret = OPENSSL_malloc(11);
+    ret = malloc(11);
     if (!ret)
         return NULL;
 
@@ -1076,7 +1074,6 @@ certificate_description_print(BIO *bio,
         const CVC_CERTIFICATE_DESCRIPTION *desc, int indent)
 {
     ASN1_OCTET_STRING *s;
-    const unsigned char *p;
     int ret, nid, count, i;
 
     if (desc == NULL)
@@ -1125,25 +1122,10 @@ certificate_description_print(BIO *bio,
 
     nid = OBJ_obj2nid(desc->descriptionType);
     if (nid == NID_id_plainFormat) {
-#ifndef HAVE_PATCHED_OPENSSL
-            ASN1_UTF8STRING *s = NULL;
-            if (desc->termsOfUsage.other->type != V_ASN1_SEQUENCE)
-                return 0;
-            p = desc->termsOfUsage.other->value.sequence->data;
-            if (!d2i_ASN1_UTF8STRING(&s, &p,
-                        desc->termsOfUsage.other->value.sequence->length))
-                return 0;
-            p = s->data;
-#else
-            p = desc->termsOfUsage.plainTerms->data;
-#endif
             if (!BIO_indent(bio, indent, 80)
-                    || !BIO_printf(bio, "%s\n%s\n", cert_desc_field_strings[5],
-                        p))
+                    || !BIO_printf(bio, "%s\n%.*s\n", cert_desc_field_strings[5],
+                        desc->termsOfUsage->length, desc->termsOfUsage->data))
                 return 0;
-#ifndef HAVE_PATCHED_OPENSSL
-            ASN1_UTF8STRING_free(s);
-#endif
             ret = 1;
     } else if (nid == NID_id_htmlFormat) {
         ret = 2;
@@ -1266,6 +1248,7 @@ static int CVC_eckey2pubkey(int all_parameters,
 
     check(out && key && bn_ctx, "Invalid Arguments");
 
+    BN_CTX_start(bn_ctx);
     ec = EVP_PKEY_get1_EC_KEY(key);
     check(ec, "Could not get EC key");
 
@@ -1329,16 +1312,11 @@ static int CVC_eckey2pubkey(int all_parameters,
 err:
     if (ec)
         EC_KEY_free(ec);
-    if (bn)
-        BN_free(bn);
-    if (a_bn)
-        BN_free(a_bn);
-    if (b_bn)
-        BN_free(b_bn);
     if (Y_buf)
         BUF_MEM_free(Y_buf);
     if (G_buf)
         BUF_MEM_free(G_buf);
+    BN_CTX_end(bn_ctx);
 
     return ok;
 }
@@ -1429,6 +1407,8 @@ CVC_pkey2pubkey(int all_parameters, int protocol, EVP_PKEY *key,
 
     if (!bn_ctx) {
         tmp_ctx = BN_CTX_new();
+        if (!tmp_ctx)
+            goto err;
         bn_ctx = tmp_ctx;
     }
 
@@ -1449,6 +1429,7 @@ CVC_pkey2pubkey(int all_parameters, int protocol, EVP_PKEY *key,
                 goto err;
             break;
         case EVP_PKEY_DH:
+        case EVP_PKEY_DHX:
             if (!CVC_dh2pubkey(all_parameters, key, bn_ctx, pubkey))
                 goto err;
             break;
@@ -1463,7 +1444,7 @@ CVC_pkey2pubkey(int all_parameters, int protocol, EVP_PKEY *key,
     out = pubkey;
 
 err:
-    if (!tmp_ctx)
+    if (tmp_ctx)
         BN_CTX_free(tmp_ctx);
     if (!out && !in && pubkey)
         CVC_PUBKEY_free(pubkey);
@@ -1501,7 +1482,7 @@ EAC_ec_key_from_asn1(EC_KEY **key, ASN1_OCTET_STRING *p, ASN1_OCTET_STRING *a,
     order_bn = BN_CTX_get(lcl_bn_ctx);
     cofactor_bn = BN_CTX_get(lcl_bn_ctx);
 
-    if (!cofactor_bn)
+    if (!p_bn || !a_bn || !b_bn || !order_bn || !cofactor_bn)
         goto err;
 
     /* Copy field and curve */
