@@ -321,6 +321,10 @@ err:
     return NULL;
 }
 
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+#include <openssl/provider.h>
+#endif
+
 BUF_MEM *
 retail_mac_des(const BUF_MEM * key, const BUF_MEM * in)
 {
@@ -331,6 +335,11 @@ retail_mac_des(const BUF_MEM * key, const BUF_MEM * in)
 
     check(key, "Invalid arguments");
 
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+    OSSL_PROVIDER *legacy;
+    legacy = OSSL_PROVIDER_load(NULL, "legacy");
+#endif
+
     len = EVP_CIPHER_block_size(EVP_des_cbc());
     check(key->length >= 2*len, "Key too short");
 
@@ -369,6 +378,9 @@ retail_mac_des(const BUF_MEM * key, const BUF_MEM * in)
     BUF_MEM_free(c_tmp);
     BUF_MEM_free(d_tmp);
     EVP_CIPHER_CTX_free(ctx);
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+    OSSL_PROVIDER_unload(legacy);
+#endif
 
     return mac;
 
@@ -381,6 +393,10 @@ err:
         BUF_MEM_free(d_tmp);
     if (ctx)
         EVP_CIPHER_CTX_free(ctx);
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+    if (legacy)
+        OSSL_PROVIDER_unload(legacy);
+#endif
 
     return NULL;
 }
@@ -601,85 +617,6 @@ authenticate(const KA_CTX *ctx, const BUF_MEM *data)
     }
 }
 
-EVP_PKEY *
-EVP_PKEY_dup(EVP_PKEY *key)
-{
-    EVP_PKEY *out = NULL;
-    DH *dh_in = NULL, *dh_out = NULL;
-    EC_KEY *ec_in = NULL, *ec_out = NULL;
-    RSA *rsa_in = NULL, *rsa_out = NULL;
-
-    check(key, "Invalid arguments");
-
-    out = EVP_PKEY_new();
-
-    if (!out)
-        goto err;
-
-    switch (EVP_PKEY_base_id(key)) {
-        case EVP_PKEY_DH:
-        case EVP_PKEY_DHX:
-            dh_in = EVP_PKEY_get1_DH(key);
-            if (!dh_in)
-                goto err;
-
-            dh_out = DHparams_dup_with_q(dh_in);
-            if (!dh_out)
-                goto err;
-
-            EVP_PKEY_set1_DH(out, dh_out);
-            DH_free(dh_out);
-            DH_free(dh_in);
-            break;
-
-        case EVP_PKEY_EC:
-            ec_in = EVP_PKEY_get1_EC_KEY(key);
-            if (!ec_in)
-                goto err;
-
-            ec_out = EC_KEY_dup(ec_in);
-            if (!ec_out)
-                goto err;
-
-            EVP_PKEY_set1_EC_KEY(out, ec_out);
-            EC_KEY_free(ec_out);
-            EC_KEY_free(ec_in);
-            break;
-
-        case EVP_PKEY_RSA:
-            rsa_in = EVP_PKEY_get1_RSA(key);
-            if (!rsa_in)
-                goto err;
-
-            rsa_out = RSAPrivateKey_dup(rsa_in);
-            if (!rsa_out)
-                goto err;
-
-            EVP_PKEY_set1_RSA(out, rsa_out);
-            RSA_free(rsa_out);
-            RSA_free(rsa_in);
-            break;
-
-        default:
-            log_err("Unknown protocol");
-            goto err;
-    }
-
-    return out;
-
-err:
-    if (dh_in)
-        DH_free(dh_in);
-    if (ec_in)
-        EC_KEY_free(ec_in);
-    if (rsa_in)
-        RSA_free(rsa_in);
-    if (out)
-        EVP_PKEY_free(out);
-
-    return NULL;
-}
-
 static EVP_PKEY *
 EVP_PKEY_from_pubkey(EVP_PKEY *key, const BUF_MEM *pub, BN_CTX *bn_ctx)
 {
